@@ -162,7 +162,7 @@ static struct Fsm plci_fsm = { 0, 0, 0, 0, 0 };
 
 static void lPLCI_debug(struct FsmInst *fi, char *fmt, ...)
 {
-	char tmp[128];
+	char tmp[160];
 	char *p = tmp;
 	va_list args;
 	struct lPLCI *lp = fi->userdata;
@@ -170,7 +170,7 @@ static void lPLCI_debug(struct FsmInst *fi, char *fmt, ...)
 	if (!(MIDEBUG_STATES & mI_debug_mask))
 		return;
 	va_start(args, fmt);
-	p += sprintf(p, "lPLCI 0x%x: ", lp->plci);
+	p += sprintf(p, "lPLCI %04x: ", lp->plci);
 	p += vsprintf(p, fmt, args);
 	*p = 0;
 	dprint(MIDEBUG_STATES, "%s\n", tmp);
@@ -1599,6 +1599,7 @@ int lPLCICreate(struct lPLCI **lpp, struct lController *lc, struct mPLCI *plci)
 	lp->plci_m.printdebug = lPLCI_debug;
 	lp->chid.nr = MI_CHAN_NONE;
 	lp->autohangup = 1;
+	pthread_mutex_init(&lp->lock, NULL);
 	*lpp = lp;
 	return 0;
 }
@@ -1607,6 +1608,7 @@ void lPLCI_free(struct lPLCI *lp)
 {
 	struct mNCCI *nc, *nn;
 
+	dprint(MIDEBUG_PLCI, "PLCI:%04x free now\n", lp->plci);
 	if (lp->BIlink)
 		CloseBInstance(lp->BIlink);
 	if (lp->PLCI) {
@@ -1636,6 +1638,7 @@ void lPLCI_free(struct lPLCI *lp)
 void lPLCIRelease(struct lPLCI *lp)
 {
 	/* TODO clean NCCIs */
+	dprint(MIDEBUG_PLCI, "PLCI:%04x plci state %s - %s NCCIs\n", lp->plci, str_st_plci[lp->plci_m.state], lp->Nccis ? "have" : "no");
 	FsmEvent(&lp->plci_m, EV_AP_RELEASE, NULL);
 }
 
@@ -1780,21 +1783,37 @@ struct mNCCI *getNCCI4addr(struct lPLCI *lp, uint32_t addr, int mode)
 void lPLCIDelNCCI(struct mNCCI *ncci)
 {
 	struct lPLCI *lp = ncci->lp;
+	struct mNCCI *nc, *onc;
 
-	if (lp->Nccis != ncci) {
-		eprint("lPLCI %04x do not cover NCCI %06x\n", lp->plci, ncci->ncci);
-		return;
+	pthread_mutex_lock(&lp->lock);
+	nc = lp->Nccis;
+	onc = NULL;
+	while (nc) {
+		if (nc == ncci) {
+			lp->NcciCnt--;
+			if (onc)
+				onc->next = nc->next;
+			break;
+		}
+		onc = nc;
+		nc = nc->next;
 	}
-	lp->Nccis = NULL;
-	lp->NcciCnt = 0;
+	if (lp->Nccis ==  ncci)
+		lp->Nccis = ncci->next;
 
-	if (ncci->BIlink) {
-		ncci->BIlink->b3data = NULL;
-		if (ncci->BIlink->fd > 0)
-			CloseBInstance(ncci->BIlink);
-		ncci->BIlink = NULL;
-		lp->BIlink = NULL;
+	if (lp->Nccis || lp->NcciCnt) {
+		iprint("PLCI:%04x still %d NCCIs do not shutdown link\n", lp->plci, lp->NcciCnt);
+	} else {
+		dprint(MIDEBUG_PLCI, "PLCI:%04x all NCCIs gone %s BIlink\n", lp->plci, ncci->BIlink ? "shutdown" : "no");
+		if (ncci->BIlink) {
+			ncci->BIlink->b3data = NULL;
+			if (ncci->BIlink->fd > 0)
+				CloseBInstance(ncci->BIlink);
+			ncci->BIlink = NULL;
+			lp->BIlink = NULL;
+		}
 	}
+	pthread_mutex_unlock(&lp->lock);
 }
 
 void B3ReleaseLink(struct lPLCI *lp, struct BInstance *bi)
@@ -2077,6 +2096,7 @@ static int lPLCILinkDown(struct lPLCI *lp)
 	struct BInstance *bi;
 
 	nc = lp->Nccis;
+	dprint(MIDEBUG_PLCI, "PLCI:%04x LinkDown %s Nccis %d\n", lp->plci, lp->Nccis ? "" : "no", lp->NcciCnt);
 	if (nc) {
 		while (nc) {
 			nn = nc->next;
